#pragma once

#include "OSAbstract/OSAbstract.h"

#define XI_006AEPRO_VENDOR_ID		0x1172
#define XI_006AEPRO_DEVICE_ID		0xA006
#define XI006AEPRO_INFO 			"XI006AEPRO"

typedef struct _XI006AEDevice XI006AEDevice;
typedef enum _XI006AE_AUDIO_INDEX {
	XI006AE_INDEX_A1,
	XI006AE_INDEX_A2,
	XI006AE_INDEX_A3,
	XI006AE_INDEX_A4,
	XI006AE_INDEX_A5,
	XI006AE_INDEX_A6,
	XI006AE_NUM_AUDIO_DEVICES
} XI006AE_AUDIO_INDEX;

typedef enum _XI006AE_VIDEO_INDEX {
	XI006AE_INDEX_V1,
	XI006AE_INDEX_V2,
	XI006AE_INDEX_V3,
	XI006AE_INDEX_V4,
	XI006AE_INDEX_V5,
	XI006AE_INDEX_V6,
	XI006AE_NUM_VIDEO_DEVICES
} XI006AE_VIDEO_INDEX;

//////////////////////////////////////////////////////
//// XI006AEDevice Implement interface
XI006AEDevice *XI006AEDevice_Create(void);
void XI006AEDevice_Destroy(XI006AEDevice *pXI006AEDevice);
int XI006AEDevice_OnPowerUp(XI006AEDevice *pXI006AEDevice,  uint8_t *bmmio, void *pci);
uint8_t * XI006AEDevice_OnPowerDown(XI006AEDevice *pXI006AEDevice);
bool XI006AEDevice_OnInterrupt(XI006AEDevice *pXI006AEDevice);
void * XI006AEPRO_GetV4l2ListHead(XI006AEDevice *pXI006AEDevice);
void * XI006AEPRO_GetAlsaListHead(XI006AEDevice *pXI006AEDevice);
unsigned char *XI006AEPRO_GetROMID(XI006AEDevice *pXI006AEDevice, unsigned int * length);

//////////////////////////////////////////////////////
//Video Implement
void XI006AEPRO_GetInputSignal(
			XI006AEDevice *pXI006AEDevice, 
			int32_t channal,
			long *lImageFormatVersion, 
			bool* bInterlaced, 
			int* cx, 
			int* cy, 
			VINEX_FORMAT* format, 
			bool* bPrescaled, 
			int* cxScaled, 
			bool *bHasSignal
			);
int XI006AEPRO_SetVideoShowInfo(
			XI006AEDevice *pXI006AEDevice, 
			int32_t channal, 
			bool bBottomUp, 
			int csOutput, 
			long width, 
			long height
			);
int XI006AEPRO_WaitForNewImage(XI006AEDevice *pXI006AEDevice, int32_t channal, uint32_t dwTimeoutJiff);
int XI006AEPRO_ProcessOneFrame(XI006AEDevice *pXI006AEDevice, int32_t channal, uint8_t * pbDestBuf);

///////////////////////////////////////////////////////////
//// Implement interface for Image Pipe
bool XI006AEPRO_IsVerticalFlip(XI006AEDevice *pXI006AEDevice, int32_t channal);
void XI006AEPRO_SetHorizontalFlip(XI006AEDevice *pXI006AEDevice, int32_t channal, bool flip);
void XI006AEPRO_SetVerticalFlip(XI006AEDevice *pXI006AEDevice, int32_t channal, bool flip);
void XI006AEPRO_SetDeinterlaceMethod(XI006AEDevice *pXI006AEDevice, int32_t channal, int method);
void XI006AEPRO_SetScaleMode(XI006AEDevice *pXI006AEDevice, int32_t channal, int mode);

///////////////////////////////////////////////////////////
//// Implement interface for Image Signal
void XI006AEPRO_GetSignalStatus(XI006AEDevice *pXI006AEDevice, int32_t channal, HDCAPTURE_SIGNAL_STATUS_S * pStatus);
void XI006AEPRO_GetVideoTimingList(
				XI006AEDevice *pXI006AEDevice, 
				int32_t channal, 
				HDCAPTURE_TIMING_LIST_S * pVideoTimingList
				);
void XI006AEPRO_SetVideoTiming(XI006AEDevice *pXI006AEDevice, int32_t channal, const HDCAPTURE_TIMING_S * pTiming);
void XI006AEPRO_GetDefaultVideoTiming(XI006AEDevice *pXI006AEDevice, int32_t channal, HDCAPTURE_DEFAULT_TIMING_S * pTiming);
void XI006AEPRO_GetVideoTiming(XI006AEDevice *pXI006AEDevice, int32_t channal,  HDCAPTURE_TIMING_S * pTiming);
void XI006AEPRO_SetFormat(XI006AEDevice *pXI006AEDevice, int32_t channal, VINEX_FORMAT format);
void XI006AEPRO_SetPrescaleParams(
				XI006AEDevice *pXI006AEDevice, 
				int32_t channal, 
				bool bEnablePrescale, 
				int cxOut, 
				int cyOut, 
				SCALEP_TYPE scaleMode
				);
VINEX_FORMAT XI006AEPRO_GetFormat(XI006AEDevice *pXI006AEDevice, int32_t channal);
bool XI006AEPRO_GetPresetTiming(XI006AEDevice *pXI006AEDevice, int32_t channal, HDCAPTURE_PRESET_TIMING_S * pPreset);
void XI006AEPRO_SetPresetTiming(XI006AEDevice *pXI006AEDevice, int32_t channal, const HDCAPTURE_PRESET_TIMING_S * pPreset);

////////////////////////////////////////////////////////////////////////////////
//// Video Paramter of Display Interface
void XI006AEPRO_VSetVideoStandardMode(
			XI006AEDevice *pXI006AEDevice, 
			int32_t channal, 
			ADV7180_VIDEO_STANDARD_MODE vidStandard
			);
ADV7180_VIDEO_STANDARD_MODE XI006AEPRO_VGetVideoStandardMode(XI006AEDevice *pXI006AEDevice, int32_t channal);

bool XI006AEPRO_VIsSquarePixelMode(XI006AEDevice *pXI006AEDevice, int32_t channal);
void XI006AEPRO_VSetSquareMode(XI006AEDevice *pXI006AEDevice, int32_t channal, bool bSquareMode);
void XI006AEPRO_VSetPreDenoiseThreshold(XI006AEDevice *pXI006AEDevice, int32_t channal, uint8_t byDenoiseThreshold);
void XI006AEPRO_VSetPostDenoiseThreshold(XI006AEDevice *pXI006AEDevice, int32_t channal, uint8_t byDenoiseThreshold);
void XI006AEPRO_VSetLumaPeakingGain(XI006AEDevice *pXI006AEDevice, int32_t channal, uint8_t byPeakingGain);

void XI006AEPRO_VSetAGCMode(
			XI006AEDevice *pXI006AEDevice, 
			int32_t channal, 
			int lumaAGCMode, 
			int chromaAGCMode
			);
void XI006AEPRO_VSetLumaGain(XI006AEDevice *pXI006AEDevice, int32_t channal, int nGain);
void XI006AEPRO_VSetChromaGain(XI006AEDevice *pXI006AEDevice, int32_t channal, int nGain);
int  XI006AEPRO_VGetLumaGain(XI006AEDevice *pXI006AEDevice, int32_t channal);
int  XI006AEPRO_VGetChromaGain(XI006AEDevice *pXI006AEDevice, int32_t channal);

void XI006AEPRO_VSetContrast(XI006AEDevice *pXI006AEDevice, int32_t channal, int nContrast);
void XI006AEPRO_VSetBrightness(XI006AEDevice *pXI006AEDevice, int32_t channal, int nBrightness);
void XI006AEPRO_VSetHue(XI006AEDevice *pXI006AEDevice, int32_t channal, int nHue);
void XI006AEPRO_VSetSaturation(XI006AEDevice *pXI006AEDevice, int32_t channal, int nSaturation);
void XI006AEPRO_VSetSharpness(XI006AEDevice *pXI006AEDevice, int32_t channal, int nSharpness);
void XI006AEPRO_VSetDefaultConfig(XI006AEDevice *pXI006AEDevice, int32_t channal);

////////////////////////////////////////////////////////////////////////////////
//// Audio Implement Interface
void XI006AEPRO_AStart(XI006AEDevice *pXI006AEDevice, int32_t channal, uint8_t *pOSRingBufBase, uint32_t cbOSRingBufSize);
void XI006AEPRO_AStop(XI006AEDevice *pXI006AEDevice, int32_t channal);
uint32_t XI006AEPRO_ACapture(XI006AEDevice *pXI006AEDevice, int32_t channal);
uint32_t XI006AEPRO_AGetOSRingBufWPos(XI006AEDevice *pXI006AEDevice, int32_t channal);
uint32_t XI006AEPRO_ADefaultOSRingBufSize(XI006AEDevice *pXI006AEDevice, int32_t channal);
uint8_t XI006AEPRO_AGetMaxVolume(void);

////////////////////////////////////////////////////////////////////////////////
//// Audio Control Implement Interface
long XI006AEPRO_GetVolume(XI006AEDevice *pXI006AEDevice, int channal, bool left);
void XI006AEPRO_SetVolume(XI006AEDevice *pXI006AEDevice, int channal, bool left, long value);
bool XI006AEPRO_GetMute(XI006AEDevice *pXI006AEDevice, int channall, bool left);
void XI006AEPRO_SetMute(XI006AEDevice *pXI006AEDevice, int channal, bool left, bool mute);
