#pragma once

#include "OSAbstract/OSAbstract.h"

#define XI_100XE_VENDOR_ID					0x1172
#define XI_100XE_DEVICE_ID					0xD101

#define XI100DE_AUDIO_DEVICE_COUNT			1
#define XI100DE_VIDEO_DEVICE_COUNT			1

typedef enum _SDI_DEVICE_INDEX {
	INDEX_SDIDevice1,
	NUM_SDI_DEVICES
}SDI_DEVICE_INDEX;

typedef struct _XI100DESDIDevice XI100DESDIDevice;

XI100DESDIDevice *XI100DESDIDevice_Create(void);
void XI100DESDIDevice_Destroy(XI100DESDIDevice *pXI100DESDIDeviceHD);
int XI100DESDIDevice_OnPowerUp(XI100DESDIDevice * pXI100DESDIDevice , uint8_t *bmmio, void *pci);
unsigned char * XI100DESDIDevice_OnPowerDown(XI100DESDIDevice * pXI100DESDIDevice , void *pci);
bool XI100DESDIDevice_OnInterrupt(XI100DESDIDevice * pXI100DESDIDevice);
void * XI100DESDIDevice_GetAlsaListHead(XI100DESDIDevice * pXI100DESDIDevice);
void * XI100DESDIDevice_GetV4l2ListHead(XI100DESDIDevice * pXI100DESDIDevice);
unsigned char * XI100DESDIDevice_GetROMID(XI100DESDIDevice *pXI100DESDIDevice, unsigned int * length);
bool XI100DESDIDevice_IsDeviceValid(XI100DESDIDevice *pXI100DESDIDevice);

////////////////////////////////////////////////////////////////////////////////
//// Video Implement interface
void XI100DESDIDevice_GetInputSignal(
			XI100DESDIDevice *pXI100DESDIDevice, 
			SDI_DEVICE_INDEX index,
			long *lImageFormatVersion, 
			bool* bInterlaced, 
			int* cx, 
			int* cy, 
			VINEX_FORMAT* format, 
			bool* bPrescaled, 
			int* cxScaled, 
			bool *bHasSignal
			);
int XI100DESDIDevice_SetVideoShowInfo(
				XI100DESDIDevice *pXI100DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				bool bBottomUp, 
				int csOutput, 
				long width, 
				long height
				);
int XI100DESDIDevice_WaitForNewImage(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, uint32_t dwTimeoutJiff);
int XI100DESDIDevice_ProcessOneFrame(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, uint8_t * pbDestBuf);
////////////////////////////////////////////////////////////////////////////////
//// Audio Implement interface
int XI100DESDIDevice_AudioCaptureStart(
				XI100DESDIDevice *pXI100DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				uint8_t * pOSRingBufBase, 
				uint32_t cbOSRingBufSize
				);
int XI100DESDIDevice_AudioCaptureStop(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI100DESDIDevice_AudioCapture(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI100DESDIDevice_AudioGetOSRingBufWPos(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI100DESDIDevice_AudioCaptureDefaultOSRingBufSize(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);

////////////////////////////////////////////////////////////////////////////////
//// interface ioctl
long XI100DESDIDevice_GetVolume(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool left);
long XI100DESDIDevice_SetVolume(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI100DESDIDevice_GetMute(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool left);
int XI100DESDIDevice_SetMute(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI100DESDIDevice_IsAudioSampleRateChanged(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_GetAudioSampleRate(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_GetSignalInfo(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index,VIDEO_SIGNAL_INFO *pSignal);
int XI100DESDIDevice_SetDeinterlaceMethod(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI100DESDIDevice_SetScaleMode(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI100DESDIDevice_IsVerticalFlip(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_SetVerticalFlip(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
int XI100DESDIDevice_IsHorizontalFlip(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_SetHorizontalFlip(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
long XI100DESDIDevice_GetBrightness(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetBrightness(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetContrast(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetContrast(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetHue(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetHue(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetSaturation(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetSaturation(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetSharpness(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetSharpness(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetPhase(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetPhase(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI100DESDIDevice_GetFormat(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetFormat(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, VINEX_FORMAT format);
long XI100DESDIDevice_SetAutoDetectCaptureMode(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool bAutoDetect);
long XI100DESDIDevice_GetAutoDetectCaptureMode(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
long XI100DESDIDevice_SetCaptureMode(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index , HDCAPTURE_MODE mode);
long XI100DESDIDevice_GetCaptureMode(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_TriggerAutoPhase(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_TriggerAutoPosition(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_IsHDCPSignal(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index);
int XI100DESDIDevice_HDCPIsOn(XI100DESDIDevice *pXI100DESDIDevice, SDI_DEVICE_INDEX index, bool on);
