#pragma once

#include "OSAbstract/OSAbstract.h"

#define XI200DE_VENDOR_ID					0x1172
#define XI200DE_DEVICE_ID					0xD202

#define XI200DE_AUDIO_DEVICE_COUNT			2
#define XI200DE_VIDEO_DEVICE_COUNT			2

typedef enum _SDI_DEVICE_INDEX {
	INDEX_SDIDevice1,
	INDEX_SDIDevice2,
	NUM_SDI_DEVICES
}SDI_DEVICE_INDEX;

typedef struct _XI200DESDIDevice XI200DESDIDevice;

XI200DESDIDevice *XI200DESDIDevice_Create(void);
void XI200DESDIDevice_Destroy(XI200DESDIDevice *pXI200DESDIDeviceHD);
int XI200DESDIDevice_OnPowerUp(XI200DESDIDevice * pXI200DESDIDevice , uint8_t *bmmio, void *pci);
unsigned char * XI200DESDIDevice_OnPowerDown(XI200DESDIDevice * pXI200DESDIDevice , void *pci);
bool XI200DESDIDevice_OnInterrupt(XI200DESDIDevice * pXI200DESDIDevice);
void * XI200DESDIDevice_GetAlsaListHead(XI200DESDIDevice * pXI200DESDIDevice);
void * XI200DESDIDevice_GetV4l2ListHead(XI200DESDIDevice * pXI200DESDIDevice);
unsigned char * XI200DESDIDevice_GetROMID(XI200DESDIDevice *pXI200DESDIDevice, unsigned int * length);
bool XI200DESDIDevice_IsDeviceValid(XI200DESDIDevice *pXI200DESDIDevice);

////////////////////////////////////////////////////////////////////////////////
//// Video Implement interface
void XI200DESDIDevice_GetInputSignal(
			XI200DESDIDevice *pXI200DESDIDevice, 
			SDI_DEVICE_INDEX index,
			long *lImageFormatVersion, 
			bool* bInterlaced, 
			int* cx, 
			int* cy, 
			VINEX_FORMAT* format, 
			bool* bPrescaled, 
			int* cxScaled, 
			bool *bHasSignal
			);
int XI200DESDIDevice_SetVideoShowInfo(
				XI200DESDIDevice *pXI200DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				bool bBottomUp, 
				int csOutput, 
				long width, 
				long height
				);
int XI200DESDIDevice_WaitForNewImage(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, uint32_t dwTimeoutJiff);
int XI200DESDIDevice_ProcessOneFrame(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, uint8_t * pbDestBuf);
////////////////////////////////////////////////////////////////////////////////
//// Audio Implement interface
int XI200DESDIDevice_AudioCaptureStart(
				XI200DESDIDevice *pXI200DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				uint8_t * pOSRingBufBase, 
				uint32_t cbOSRingBufSize
				);
int XI200DESDIDevice_AudioCaptureStop(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI200DESDIDevice_AudioCapture(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI200DESDIDevice_AudioGetOSRingBufWPos(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI200DESDIDevice_AudioCaptureDefaultOSRingBufSize(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);

////////////////////////////////////////////////////////////////////////////////
//// interface ioctl
long XI200DESDIDevice_GetVolume(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool left);
long XI200DESDIDevice_SetVolume(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI200DESDIDevice_GetMute(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool left);
int XI200DESDIDevice_SetMute(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI200DESDIDevice_IsAudioSampleRateChanged(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_GetAudioSampleRate(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_GetSignalInfo(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index,VIDEO_SIGNAL_INFO *pSignal);
int XI200DESDIDevice_SetDeinterlaceMethod(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI200DESDIDevice_SetScaleMode(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI200DESDIDevice_IsVerticalFlip(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_SetVerticalFlip(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
int XI200DESDIDevice_IsHorizontalFlip(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_SetHorizontalFlip(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
long XI200DESDIDevice_GetBrightness(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetBrightness(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetContrast(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetContrast(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetHue(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetHue(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetSaturation(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetSaturation(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetSharpness(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetSharpness(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetPhase(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetPhase(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI200DESDIDevice_GetFormat(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetFormat(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, VINEX_FORMAT format);
long XI200DESDIDevice_SetAutoDetectCaptureMode(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool bAutoDetect);
long XI200DESDIDevice_GetAutoDetectCaptureMode(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
long XI200DESDIDevice_SetCaptureMode(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index , HDCAPTURE_MODE mode);
long XI200DESDIDevice_GetCaptureMode(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_TriggerAutoPhase(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_TriggerAutoPosition(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_IsHDCPSignal(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index);
int XI200DESDIDevice_HDCPIsOn(XI200DESDIDevice *pXI200DESDIDevice, SDI_DEVICE_INDEX index, bool on);
