#pragma once

#include "OSAbstract/OSAbstract.h"

#define XI400DE_VENDOR_ID					0x1172
#define XI400DE_DEVICE_ID					0xD401
#define XI400DE_AUDIO_DEVICE_COUNT			4
#define XI400DE_VIDEO_DEVICE_COUNT			4

typedef enum _SDI_DEVICE_INDEX {
	INDEX_SDIDevice1,
	INDEX_SDIDevice2,
	INDEX_SDIDevice3,
	INDEX_SDIDevice4,
	NUM_SDI_DEVICES
}SDI_DEVICE_INDEX;

typedef struct _XI400DESDIDevice XI400DESDIDevice;

XI400DESDIDevice *XI400DESDIDevice_Create(void);
void XI400DESDIDevice_Destroy(XI400DESDIDevice *pXI400DESDIDeviceHD);
int XI400DESDIDevice_OnPowerUp(XI400DESDIDevice * pXI400DESDIDevice , uint8_t *bmmio, void *pci);
unsigned char * XI400DESDIDevice_OnPowerDown(XI400DESDIDevice * pXI400DESDIDevice , void *pci);
bool XI400DESDIDevice_OnInterrupt(XI400DESDIDevice * pXI400DESDIDevice);
void * XI400DESDIDevice_GetAlsaListHead(XI400DESDIDevice * pXI400DESDIDevice);
void * XI400DESDIDevice_GetV4l2ListHead(XI400DESDIDevice * pXI400DESDIDevice);
unsigned char * XI400DESDIDevice_GetROMID(XI400DESDIDevice *pXI400DESDIDevice, unsigned int * length);
bool XI400DESDIDevice_IsDeviceValid(XI400DESDIDevice *pXI400DESDIDevice);

////////////////////////////////////////////////////////////////////////////////
//// Video Implement interface
void XI400DESDIDevice_GetInputSignal(
			XI400DESDIDevice *pXI400DESDIDevice, 
			SDI_DEVICE_INDEX index,
			long *lImageFormatVersion, 
			bool* bInterlaced, 
			int* cx, 
			int* cy, 
			VINEX_FORMAT* format, 
			bool* bPrescaled, 
			int* cxScaled, 
			bool *bHasSignal
			);
int XI400DESDIDevice_SetVideoShowInfo(
				XI400DESDIDevice *pXI400DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				bool bBottomUp, 
				int csOutput, 
				long width, 
				long height
				);
int XI400DESDIDevice_WaitForNewImage(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, uint32_t dwTimeoutJiff);
int XI400DESDIDevice_ProcessOneFrame(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, uint8_t * pbDestBuf);
////////////////////////////////////////////////////////////////////////////////
//// Audio Implement interface
int XI400DESDIDevice_AudioCaptureStart(
				XI400DESDIDevice *pXI400DESDIDevice, 
				SDI_DEVICE_INDEX index, 
				uint8_t * pOSRingBufBase, 
				uint32_t cbOSRingBufSize
				);
int XI400DESDIDevice_AudioCaptureStop(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI400DESDIDevice_AudioCapture(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI400DESDIDevice_AudioGetOSRingBufWPos(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
uint32_t XI400DESDIDevice_AudioCaptureDefaultOSRingBufSize(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);

////////////////////////////////////////////////////////////////////////////////
//// interface ioctl
long XI400DESDIDevice_GetVolume(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool left);
long XI400DESDIDevice_SetVolume(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI400DESDIDevice_GetMute(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool left);
int XI400DESDIDevice_SetMute(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool left, long value);
int XI400DESDIDevice_IsAudioSampleRateChanged(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_GetAudioSampleRate(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_GetSignalInfo(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index,VIDEO_SIGNAL_INFO *pSignal);
int XI400DESDIDevice_SetDeinterlaceMethod(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI400DESDIDevice_SetScaleMode(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, int method);
int XI400DESDIDevice_IsVerticalFlip(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_SetVerticalFlip(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
int XI400DESDIDevice_IsHorizontalFlip(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_SetHorizontalFlip(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool flip);
long XI400DESDIDevice_GetBrightness(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetBrightness(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetContrast(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetContrast(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetHue(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetHue(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetSaturation(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetSaturation(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetSharpness(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetSharpness(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetPhase(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetPhase(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, long val);
long XI400DESDIDevice_GetFormat(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetFormat(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, VINEX_FORMAT format);
long XI400DESDIDevice_SetAutoDetectCaptureMode(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool bAutoDetect);
long XI400DESDIDevice_GetAutoDetectCaptureMode(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
long XI400DESDIDevice_SetCaptureMode(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index , HDCAPTURE_MODE mode);
long XI400DESDIDevice_GetCaptureMode(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_TriggerAutoPhase(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_TriggerAutoPosition(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_IsHDCPSignal(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index);
int XI400DESDIDevice_HDCPIsOn(XI400DESDIDevice *pXI400DESDIDevice, SDI_DEVICE_INDEX index, bool on);
