#pragma once

#include <linux/compiler.h>
#include <linux/types.h>
#include <linux/list.h>
#include <linux/jiffies.h>
#include <linux/mutex.h>//add for I2CMasterWE
#include <asm/processor.h>
#include <asm/alternative.h>
#include <asm/cmpxchg.h>
#include <asm/param.h>
#include <asm/page.h>

#include "PrivateType.h"

////////////////////////////////////////////////////
//for hdcp
extern unsigned int xi_hdcp_off;

////////////////////////////////////////////////////
// debug
extern unsigned int xi_debug;
#define ZH_DEBUG_ENABLE
#ifdef 	ZH_DEBUG_ENABLE
	#define ZH_DEBUG(level, fmt, args...) { 		\
		if (xi_debug >= level) 						\
			OS_DebugPrint("[%d:%24s]" fmt, 			\
				 __LINE__ ,__func__, ## args); 		\
	}
#else
	#define ZH_DEBUG(level, fmt, args...) do {} while(0)
#endif

#define  ZH_DEBUG_V1 ZH_DEBUG(1,)

////////////////////////////////////////////////////
// atomic 
typedef struct _OS_ATOMIC{
       int counter;
} OS_ATOMIC;

#define ATOMIC_INIT(i)	{ (i) }

static inline 
int OS_AtomicRead(const OS_ATOMIC *v)
{
	return (*(volatile int *)&(v)->counter);
}

static inline 
void OS_AtomicSet(OS_ATOMIC *v, int i)
{
	v->counter = i;
}

static inline 
void OS_AtomicAdd(int i, OS_ATOMIC *v)
{
	asm volatile(LOCK_PREFIX "addl %1,%0"
		     : "+m" (v->counter)
		     : "ir" (i));
}

static inline 
void OS_AtomicSub(int i, OS_ATOMIC *v)
{
	asm volatile(LOCK_PREFIX "subl %1,%0"
		     : "+m" (v->counter)
		     : "ir" (i));
}

static inline 
int OS_AtomicAdd_Return(int i, atomic_t *v)
{
	int __i;

#ifdef CONFIG_M386
	unsigned long flags;
	if (unlikely(boot_cpu_data.x86 <= 3))
		goto no_xadd;
#endif
	//return i + xadd(&v->counter, i);
	
	__i = i;
	asm volatile(LOCK_PREFIX "xaddl %0, %1"
				: "+r" (i), "+m" (v->counter)
				: : "memory");
	return i + __i;

#ifdef CONFIG_M386
no_xadd: 
	raw_local_irq_save(flags);
	__i = atomic_read(v);
	atomic_set(v, i + __i);
	raw_local_irq_restore(flags);
	return i + __i;
#endif
}

static inline 
int OS_AtomicDec_Return(int i, atomic_t *v)
{
	return OS_AtomicAdd_Return(-i, v);
}

static inline 
int OS_AtomicCompareXchg(OS_ATOMIC *v, int old, int new)
{
	return cmpxchg(&v->counter, old, new);
}

static inline
int OS_AtomicXchg(OS_ATOMIC *v, int new)
{
	return xchg(&v->counter, new);
}

/////////////////////////////////////////////////////
// List will inline only
static inline 
void OS_InitListHead(struct list_head *list)
{
	INIT_LIST_HEAD(list);
}

static inline 
int OS_ListEmpty(const struct list_head *head)
{
	return list_empty(head);
}

static inline 
void OS_ListDel(struct list_head *entry)
{
	list_del(entry);
}

static inline 
void OS_ListAddTail(struct list_head *new, struct list_head *head)
{
	list_add_tail(new, head);
}

#define OS_ListEntry(ptr, type, member) list_entry(ptr, type, member)

///////////////////////////////////////
// page
static inline
int OS_PAGE_ALIGN(unsigned long addr)
{
	return (((addr)+PAGE_SIZE-1)&PAGE_MASK);
}

#define OS_PAGE_SIZE 	PAGE_SIZE
#define OS_PAGE_MASK 	PAGE_MASK
#define OS_PAGE_SHIFT	PAGE_SHIFT

////////////////////////////////////////////////////////////////////////////////
// math 
#define OS_Min(x, y)	({ 						\
						typeof(x) _x = (x);     \
						typeof(y) _y = (y);     \
						(void) (&_x == &_y);    \
						_x < _y ? _x : _y; })
						
#define OS_Max(x, y) 	({ 						\
						typeof(x) _x = (x);     \
						typeof(y) _y = (y);     \
						(void) (&_x == &_y);    \
						_x > _y ? _x : _y; })						

////////////////////////////////////////////////////////////////////////////////
// wait queue
typedef struct _OS_WaitQueueHeadT OS_WaitQueueHeadT;
OS_WaitQueueHeadT * OS_WaitQueueHeadTCreate(void);
void OS_WaitQueueHeadTDestroy(OS_WaitQueueHeadT * t);

void OS_InitWaitQueueHead(OS_WaitQueueHeadT *q);
int OS_WaitEventInterruptible(OS_WaitQueueHeadT * wq, unsigned int condition);
int OS_WaitEventInterruptibleTimeout(OS_WaitQueueHeadT * wq, unsigned int condition, int jtime);
void OS_WakeUpInterruptible(OS_WaitQueueHeadT *wq);

////////////////////////////////////////////////////////////////////////////////
// Debug
void OS_DebugPrint(const char * pszFormat, ...);
void OS_BreakOn(bool bBreak, const char * pszFormat, ...);
long OS_ISERR(const void *ptr);
long OS_PTRERR(const void *ptr);

////////////////////////////////////////////////////////////////////////////////
// Memory
void * OS_Alloc(uint32_t cbAlloc);
void * OS_AllocZero(uint32_t cbAlloc);
void OS_Free(void * pvMem);

void * OS_AlignedAlloc(uint32_t cbSize, uint32_t cbAlign);
void OS_AlignedFree(void * pvMem);

void OS_CopyMemory(void * pvDest, const void * pvSrc, uint32_t cbCopy);
void OS_MoveMemory(void * pvDest, const void * pvSrc, uint32_t cbCopy);
int OS_CompareMemory(const void * pvMem1, const void * pvMem2, uint32_t cbCompare);
void OS_InitMemory(void * pvDest, uint8_t byData, uint32_t cbInit);

////////////////////////////////////////////////////////////////////////////////
// Float pointer context protector
void OS_KernelFPUBegin(void);
void OS_KernelFPUEnd(void);

////////////////////////////////////////////////////////////////////////////////
// Delay
void OS_Sleep(int nDurationMS);
void OS_SleepUS(int nDurationUS);

////////////////////////////////////////////////////////////////////////////////
// I/O
uint32_t OS_ReadReg32(void * pvAddr);
void OS_WriteReg32(void * pvAddr, uint32_t dwData);

////////////////////////////////////////////////////////////////////////////////
// Spin Lock
typedef struct _OS_SPINLOCK OS_SPINLOCK;

OS_SPINLOCK * OS_SpinLockCreate(void);
void OS_SpinLockDestroy(OS_SPINLOCK * pSpinLock);

void OS_SpinLock(OS_SPINLOCK * pSpinLock);
void OS_SpinUnlock(OS_SPINLOCK * pSpinLock);

////////////////////////////////////////////////////////////////////////////////
// Thread
typedef struct _OS_Thread OS_Thread;
OS_Thread * OS_CreateThread(int (*threadfn)(void *data),
				   void *data,
				   const char namefmt[]);
int OS_StopThread(OS_Thread *pThread);
bool OS_IsThreadShouldStop(void);
void OS_SetFreezable(void);
void OS_TryToFreeze(void);

void *OS_DMABufferCreate(void *hwdev, size_t size, dma_addr_t *dma_handle);
void OS_DMABufferDestroy(void *hwdev, size_t size, void *vaddr, dma_addr_t dma_handle);

////////////////////////////////////////////////////////////////////////////////
//  div64
uint64_t OS_Div64(uint64_t dividend, uint64_t divisor);

////////////////////////////////////////////////////////////////////////////////
// Event 
typedef struct _OS_Completion OS_Completion;
OS_Completion * OS_CreateEvent(void);
void OS_DestroyEvent(OS_Completion * pEvent);
int OS_WaitForEvent(OS_Completion * pEvent, uint32_t dwTimeoutJiffies);
void OS_SetEvent(OS_Completion * pEvent);
void OS_ResetEvent(OS_Completion * pEvent);


////////////////////////////////////////////////////////////////////////////////
// just for test
static inline void OS_PrintBufferMap(const void* buffer, unsigned int length, unsigned width)
{
	int i,j,k;
	unsigned char *pbuf = (unsigned char*)buffer;
	
	if(!buffer || 0 == length)
		return ;
	
	if(0 == width) width = 16;
	
	j = (length + width - 1) / width;
	
	ZH_DEBUG(3, "Show Buffer Map : %d \n", length);
	for(k = 0; k < j; k++) {
		for(i = 0; i < width; i++)
			ZH_DEBUG(3, "%d", *pbuf++);
		
		ZH_DEBUG(3, "\n");
	}			
}