#include <linux/version.h>
#include <linux/init.h>
#include <linux/err.h>
#include <linux/jiffies.h>
#include <linux/slab.h>
#include <linux/time.h>
#include <linux/wait.h>
#include <linux/moduleparam.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mutex.h>
#include <linux/kthread.h>
#include <linux/types.h> 
#include <linux/fs.h> 
#include <linux/mm.h> 
#include <linux/cdev.h> 
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
#include <linux/freezer.h>
#endif
#include <asm/io.h> 
//#include <asm/system.h> 
#include <asm/uaccess.h> 

#include "Drivers/XI100XE_Device.h"
#include "audio.h"

//////////////////////////////////////////////////////////////////////////////////////
int xi_audio_start(xi_audio *drv, int bsize, int bps, int channal)
{
	if((!drv->buf) || (drv->bsize != bsize)) {
		if(drv->buf)
			vfree(drv->buf);
			
		drv->buf = vmalloc(bsize);
		if(NULL == drv->buf) {
			drv->bsize = 0;
			return -ENOMEM;
		}
			
		drv->bsize = bsize;
	}
	
	memset(drv->buf, 0x0, drv->bsize);
	XI100XEDevice_AStart(drv->dev, drv->buf, drv->bsize);	
	atomic_set(&drv->start,1);

	return 0;
}

void xi_audio_stop(xi_audio *drv)
{
	atomic_set(&drv->start,0);
	XI100XEDevice_AStop(drv->dev);
	ZH_DEBUG(1,"Stop capture \n");
}

int xi_audio_postion(xi_audio *drv)
{
	return atomic_read(&drv->wpos);
}
//////////////////////////////////////////////

static int xi_audio_open(struct inode *inode, struct file *filp) 
{ 
	xi_audio *drv;
	
	drv = container_of(inode->i_cdev, xi_audio, cdev);
	filp->private_data = drv;
	
	ZH_DEBUG(1,"Open \n");
	
	return 0; 
} 

static int xi_audio_close(struct inode *inode, struct file *filp) 
{ 
	ZH_DEBUG(1,"release \n");
	return 0; 
} 

static ssize_t xi_audio_read(struct file *filp, char __user * buf,  size_t size,  loff_t *ppos) 
{ 
	ssize_t ret = 0;
	
	xi_audio *drv = filp->private_data;
	long bsize = drv->bsize;
	int lastrpos = drv->rpos;

	if(lastrpos + size > bsize)
		size = bsize - lastrpos;

	mutex_lock(&drv->bmutex);
	ret = copy_to_user(buf, drv->buf + lastrpos, size);
	mutex_unlock(&drv->bmutex);

	size -= ret;
	drv->rpos += size;
	drv->rpos %= bsize;
	
	return size;
} 

static loff_t xi_audio_llseek(struct file *filp,loff_t offset, int whence) 
{ 
	int newpos, bsize; 
	
	xi_audio *drv = filp->private_data;

	bsize = drv->bsize;
	
	switch(whence) 
	{ 
		 case 0: 
			 newpos = offset; 
			 break; 
		 
		 case 1: 
			 newpos = filp->f_pos + offset; 
			 break; 
			 
		 case 2: 
			 newpos = bsize + offset; 
			 break; 
			 
		 default: 
			 return -EINVAL; 
	} 
	
	newpos %= bsize; 
	
	if((newpos < 0)|| (newpos > bsize)) 
		 return -EINVAL; 
		 
	filp->f_pos = newpos; 
	
	return newpos; 
} 

/*
 * only one ioctl of XI_AUDIO_IOCCTREE
 */
static int volume_line_get(xi_audio *drv, uint8_t * pl, uint8_t * pr)
{
	*pl = drv->volume[0];
	*pr = drv->volume[1];
	return 0;
}

static int volume_line_put(xi_audio *drv, uint8_t l, uint8_t r)
{
	if(l == drv->volume[0] && r == drv->volume[1])
		return 0;
		
	XI100XEDevice_SetALVolume(drv->dev, l);
	XI100XEDevice_SetARVolume(drv->dev, r);
	
	drv->volume[0] = l;
	drv->volume[1] = r;
	
	return 1;
}

static int volume_hdmi_get(xi_audio *drv, uint8_t * pl, uint8_t * pr)
{
	*pl = drv->vhdmi[0];
	*pr = drv->vhdmi[1];
	return 0;
}

static int volume_hdmi_put(xi_audio *drv, uint8_t l, uint8_t r)
{
	if(l == drv->vhdmi[0] && r == drv->vhdmi[1])
		return 0;
		
	XI100XEDevice_SetHDMILVolume(drv->dev, l);
	XI100XEDevice_SetHDMIRVolume(drv->dev, r);
	
	drv->vhdmi[0] = l;
	drv->vhdmi[1] = r;
	
	return 1;
}

long xi_audio_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	int retval = 0;
	xi_audio *drv = filp->private_data;

	if (_IOC_TYPE(cmd) != XI_AUDIO_IOC_MAGIC) 
		return -ENOTTY;
	
	if (_IOC_NR(cmd) > XI_AUDIO_IOC_MAXNR) 
		return -ENOTTY;
	
	if (_IOC_DIR(cmd) & _IOC_READ)
		retval = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
	else if (_IOC_DIR(cmd) & _IOC_WRITE)
		retval =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));

	if (retval) 
		return -EFAULT;
	
	switch(cmd) {
		case XI_AUDIO_IOCSTART : 
		{
			audio_param *param = (audio_param *)arg;
			retval = xi_audio_start(drv, param->bsize, param->bps, param->channal);
			break;
		}

		case XI_AUDIO_IOCSTOP : 
		{
			xi_audio_stop(drv);
			break;
		}

		case XI_AUDIO_IOCPOST : 
		{
			int wpos = xi_audio_postion(drv);
			retval = put_user(wpos, (int *)arg);
			break;
		}

		case XI_AUDIO_IOCGVLINE : 
		{
			uint8_t lr[2];
			volume_line_get(drv, &lr[0], &lr[1]);
			
			if(copy_to_user((uint8_t __user *)arg, lr, sizeof(lr)))
				retval = -EFAULT;
			break;
		}
		
		case XI_AUDIO_IOCGVHDMI : 
		{
			uint8_t lr[2];
			volume_hdmi_get(drv, &lr[0], &lr[1]);

			if(copy_to_user((uint8_t __user *)arg, lr, sizeof(lr)))
				retval = -EFAULT;
			break;
		}
				
		case XI_AUDIO_IOCSVLINE : 
		{
			uint8_t lr[2];
			if(copy_from_user(lr, (uint8_t __user *)arg, sizeof(lr)))
				retval = -EFAULT;

			volume_line_put(drv, lr[0], lr[1]);
			break;
		}
				
		case XI_AUDIO_IOCSVHDMI : 
		{
			uint8_t lr[2];
			if(copy_from_user(lr, (uint8_t __user *)arg, sizeof(lr)))
				retval = -EFAULT;

			volume_hdmi_put(drv, lr[0], lr[1]);
			break;
		}
				

		default: 
			return -ENOTTY;
	}
	
	return retval;
}

static const struct file_operations xi_audio_fops = { 
	.owner			= THIS_MODULE, 
	.open			= xi_audio_open, 
	.read			= xi_audio_read, 
	.release		= xi_audio_close, 
	.llseek			= xi_audio_llseek, 
	.unlocked_ioctl = xi_audio_ioctl,
}; 
/**************************************
			thread
 **************************************/
static void alsa_harvest(xi_audio *drv)
{
	int cbData, total;

	mutex_lock(&drv->bmutex);
	cbData = XI100XEDevice_ACapture(drv->dev);
	if((total = atomic_add_return(cbData, &drv->wpos)) >= drv->bsize)
		atomic_set(&drv->wpos,total - drv->bsize);
	mutex_unlock(&drv->bmutex);
}

static void audio_sleep(xi_audio *drv,int jcount)
{
	DECLARE_WAITQUEUE(wait, current);
	add_wait_queue(&drv->wq, &wait); 
	
	if ((!kthread_should_stop()) && (jcount > 0)) {
		schedule_timeout_interruptible(jcount);
	}
	remove_wait_queue(&drv->wq, &wait);	
}

static int audio_kthread(void *data)
{
	xi_audio *drv = data;
	
	ZH_DEBUG(0, "Start !\n");
	while(true) {
		if (kthread_should_stop())
			break;

		if(atomic_read(&drv->start)) 
			alsa_harvest(drv);
			
		audio_sleep(drv,HZ/100);
	}
	ZH_DEBUG(0, "Stop !\n");
	
	return 0;
}


/******************************************************************************
							module ops
******************************************************************************/
static int audio_instance_create(void *dev,int inst)
{
	int err, major = 0;
	dev_t devno = 0; 
	xi_audio  *drv;

	if(major){
		devno = MKDEV(major, 0);
		err = register_chrdev_region(devno, 1, "xi_audio");
	} else {
		err = alloc_chrdev_region(&devno, 0, 1, "xi_audio"); 
		major = MAJOR(devno); 
		printk("xi_audio major:%d\n", major); 
	} 
	
	if(err){ 
		 printk("can't get major devno:%d\n", major); 
		 goto err_alloc_chrdev; 
	} 
	
	drv = kzalloc(sizeof(xi_audio), GFP_KERNEL);
	if(drv == NULL) {
		err = -ENOMEM;
		goto err_memory;
	}
		
	cdev_init(&drv->cdev,&xi_audio_fops); 
	drv->cdev.owner= THIS_MODULE; 
	drv->cdev.ops = &xi_audio_fops;
	
	err = cdev_add(&drv->cdev, MKDEV(major, 0), 1); 
	if(err) {
		 printk("add cdev faild,err is %d\n", err); 
		 goto err_cdev_add;
	}
	
	drv->dev = dev;
	drv->channal = inst;
	drv->major = major;
	atomic_set(&drv->start,0);

	//start kthread
	init_waitqueue_head(&drv->wq);
	if(!drv->pthread) {
		drv->pthread = kthread_run(audio_kthread,drv,"XI100XE_AUDIO_THREAD");
		if (IS_ERR(drv->pthread)) {
			err = PTR_ERR(drv->pthread);
			goto error_thread;
		}
	}
		
	drv->romid = XI100XEDevice_GetROMID(dev,&drv->rlg);
	mutex_init(&drv->bmutex);
	atomic_set(&drv->wpos,0);
	drv->rpos = 0;
	
	drv->volume[0] = drv->volume[1] = 0x1F;
	drv->vhdmi[0] = drv->vhdmi[1] = 0x1F;
	
	list_add_tail(&drv->list, XI100XE_GetAlsaListHead(dev));
		
	return 0;
	
error_thread:
	cdev_del(&drv->cdev);
err_cdev_add:
	kfree(drv);
err_memory:
	unregister_chrdev_region(MKDEV(major,0), 1);
err_alloc_chrdev:
	xi_audio_release(dev);
	return err;
}

int xi_audio_release(void *dev)
{
	xi_audio  *drv;
	struct list_head *list;
	struct list_head *head;
	
	head = XI100XE_GetAlsaListHead(dev);

	while (!list_empty(head)) {
		list = head->next;
		list_del(list);
		drv = list_entry(list, xi_audio, list);
		if(drv->pthread) {
			kthread_stop(drv->pthread);
			drv->pthread = NULL;
		}
		
		cdev_del(&drv->cdev);
		
		if(drv->major)
			unregister_chrdev_region(MKDEV(drv->major,0), 1);
			
		if(drv->buf)
			vfree(drv->buf);
			
		drv->bsize = 0;
		atomic_set(&drv->wpos,0);
		drv->rpos = 0;
		
		kfree(drv);
	}
	
	ZH_DEBUG(0, "Remove AUDIO successful \n");
	
	return 0;
}

int xi_audio_init(void *dev)
{
	int i, ret = -ENOMEM;

	for (i = 0; i < XI100XE_AUDIO_DEVICE_COUNT; i++) {
		ret = audio_instance_create(dev,i);
		if (ret)
			break;
	}
	ZH_DEBUG(0, "Create %d Capture Device\n",i);
	
	return ret;
}

void xi_audio_suspend(void *dev)
{
	
}

void xi_audio_resume(void *dev)
{

}
