#pragma once

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/font.h>
#include <linux/mutex.h>
#include <linux/videodev2.h>
#include <linux/kthread.h>
#include <linux/freezer.h>
#include <media/videobuf2-vmalloc.h>
#include <media/v4l2-device.h>
#include <media/v4l2-ioctl.h>
#include <media/v4l2-ctrls.h>
#include <media/v4l2-fh.h>
#include <media/v4l2-common.h>

#include "OSAbstract/OSAbstract.h"
#include "Drivers/XI100XE_Device.h"

#define VIDEO_BUF_MAX_M		16
#define MAX_WIDTH 			1920
#define MAX_HEIGHT 			1080

typedef struct _VFmt {
	char 	*name;
	u32		fourcc;
	int 	depth;
}VFmt;

typedef enum _FrameSyscBy {
	FRAME_SYSC_BY_RATE,
	FRAME_SYSC_BY_IRQ
}FrameSyscBy;

typedef struct _VBuf {
	struct vb2_buffer	vb;
	struct list_head	buflist;
}VBuf;

typedef struct _VManufactory {
	VideoShowInfo			show;
	
	struct task_struct 		*pthread;
	struct list_head 		actives; 
	wait_queue_head_t		wq;
	int 					channal;
	void *					dev;

	unsigned long 			jbefore;
	unsigned long 			jdelay;
	unsigned long 			prevrate;
	uint32_t				framerate;

	spinlock_t				slock;
	enum v4l2_field			field;
	unsigned long			field_count;
	struct vb2_queue		vb_vidq;
}Vmanufactory;

typedef struct _XIVdrv {
	struct v4l2_device	v4l2_dev;
	Vmanufactory		vm;
	
	struct mutex		mutex;
	atomic_t			used;
	signed int			channal;
	void  *				dev;
	
	struct video_device	*vfd;

	struct v4l2_ctrl_handler	ctrl_handler;
	struct v4l2_ctrl	*square_mode;
	struct v4l2_ctrl	*agc_mode;
	struct v4l2_ctrl	*contrast;
	struct v4l2_ctrl	*brightness;
	
	struct v4l2_ctrl	*hue;
	struct v4l2_ctrl	*saturation;
	struct v4l2_ctrl	*sharpness;
	struct v4l2_ctrl	*deinterlace;
	
	struct v4l2_ctrl	*scale;
	struct v4l2_ctrl	*vflip;
	struct v4l2_ctrl	*hflip;
	
	struct v4l2_ctrl	*signal_valid;
	struct v4l2_ctrl	*signal_changed;
	struct v4l2_ctrl	*signal_width;
	struct v4l2_ctrl	*signal_height;
	struct v4l2_ctrl	*signal_fps;
	
	struct list_head	vlist;
}XIVdrv;

int xi_v4l2_init(void *dev);
int xi_v4l2_release(void *dev);
static inline void xi_v4l2_suspend(void *dev)
{
}
static inline void  xi_v4l2_resume(void *dev)
{
}
