#include <linux/module.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/random.h>
#include <linux/version.h>
#include <linux/videodev2.h>
#include <linux/dma-mapping.h>
#ifdef CONFIG_VIDEO_V4L1_COMPAT
#include <linux/videodev.h>
#endif
#include <linux/interrupt.h>
#include <media/v4l2-common.h>
#include <linux/kthread.h>
#include <linux/highmem.h>

#include <linux/fs.h>
#include "OSAbstract/PrivateType.h"
#include "v4l2.h"

static VFmt formats[] = {
	{
		.name     = "4:2:2, packed, UYVY",
		.fourcc   = V4L2_PIX_FMT_UYVY,
		.depth    = 16,
	},
		{
		.name     = "4:2:2, packed, YUYV",
		.fourcc   = V4L2_PIX_FMT_YUYV,
		.depth    = 16,
	},
};

#define XI_CID_CUSTOM_BASE		(V4L2_CID_USER_BASE | 0xe000)
#define XI_CID_CUSTOM_VFLIP  		(XI_CID_CUSTOM_BASE + 1)
#define XI_CID_CUSTOM_HFLIP		(XI_CID_CUSTOM_BASE + 2)
#define XI_CID_CUSTOM_AUTO_TRIGGER	(XI_CID_CUSTOM_BASE + 3)
#define XI_CID_CUSTOM_DEINTERLACE	(XI_CID_CUSTOM_BASE + 4)
#define XI_CID_CUSTOM_SCALE		(XI_CID_CUSTOM_BASE + 5)
#define XI_CID_CUSTOM_PHASE		(XI_CID_CUSTOM_BASE + 6)
#define XI_CID_CUSTOM_TIMING		(XI_CID_CUSTOM_BASE + 7)

const static struct v4l2_queryctrl xi_qctrl[] = {
	{
		.id            = V4L2_CID_BRIGHTNESS,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "Brightness",
		.minimum       = -128,
		.maximum       = 127,
		.step          = 1,
		.default_value = 10,
		.flags         = 0,
	}, {
		.id            = V4L2_CID_CONTRAST,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "Contrast",
		.minimum       = 0,
		.maximum       = 255,
		.step          = 1,
		.default_value = 128,
		.flags         = 0,
	}, {
		.id            = V4L2_CID_SATURATION,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "Saturation",
		.minimum       = 0,
		.maximum       = 255,
		.step          = 0x1,
		.default_value = 128,
		.flags         = 0,
	}, {
		.id            = V4L2_CID_HUE,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "Hue",
		.minimum       = -128,
		.maximum       = 127,
		.step          = 0x1,
		.default_value = 0,
		.flags         = 0,
	}, {
		.id 			= XI_CID_CUSTOM_HFLIP,
		.type 			= V4L2_CTRL_TYPE_BOOLEAN,
        .name 			= "Image HFLIP",
		.minimum 		= 0,
		.maximum 		= 1,
		.step 			= 1,
		.default_value 	= 0,
	}, {
		.id 			= XI_CID_CUSTOM_VFLIP,
		.type 			= V4L2_CTRL_TYPE_BOOLEAN,
        .name 			= "Image VFLIP",
		.minimum 		= 0,
		.maximum 		= 1,
		.step 			= 1,
		.default_value 	= 0,
	}, {
		.id            	= XI_CID_CUSTOM_AUTO_TRIGGER,
		.type          	= V4L2_CTRL_TYPE_BOOLEAN,
		.name          	= "AUTO TRIGGER",
		.minimum 		= 0,
		.maximum 		= 1,
		.step 			= 1,
		.default_value 	= 0,	
	}, {
		.id       		= XI_CID_CUSTOM_DEINTERLACE,
		.type          	= V4L2_CTRL_TYPE_MENU,
		.name          	= "DEINTERLACE",
		.minimum       	= 0,
		.maximum       	= 1,
		.step          	= 0x1,
		.default_value 	= 0,
	}, {
		.id            = XI_CID_CUSTOM_SCALE,
		.type          = V4L2_CTRL_TYPE_MENU,
		.name          = "SCALE",
		.minimum       = 0,
		.maximum       = 2,
		.step          = 0x1,
		.default_value = 0,	
	}, {
		.id            = XI_CID_CUSTOM_PHASE,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "PHASE",
		.minimum       = 0,
		.maximum       = 0x1F,
		.step          = 0x1,
		.default_value = 0,
	}, {
		.id            = XI_CID_CUSTOM_TIMING,
		.type          = V4L2_CTRL_TYPE_INTEGER,
		.name          = "TIMING",
		.minimum       = 0,
		.maximum       = 255,
		.step          = 0x1,
		.default_value = 0,
	},
};

/******************************************************************************
							misc ops
******************************************************************************/
static VFmt *get_format(struct v4l2_format *f)
{	
	unsigned int k;
		
	for (k = 0; k < ARRAY_SIZE(formats); k++) {
		if (formats[k].fourcc == f->fmt.pix.pixelformat)
			return &formats[k];
	}
	return NULL;
} 

#define time_is_after_jiffies(time) time_before(jiffies, time)
/**************************************************************
 * 					kthread ops
 **************************************************************/ 
 static void v4l2_sleep(Vmanufactory *vm,unsigned long jcount)
{
	DECLARE_WAITQUEUE(wait, current);
	add_wait_queue(&vm->wq, &wait);

	if ((!kthread_should_stop()) && (jcount > 0)) {
		schedule_timeout_interruptible(jcount);		
	}

	remove_wait_queue(&vm->wq, &wait);

	try_to_freeze();
}

static void v4l2_framerate_adjust(Vmanufactory *vm, bool runing)
{
	unsigned long jnext;
	if(!runing) {
		vm->run = false;
		vm->prevrate = 0;
		return ;
	}
	
	if(!vm->run)
		vm->run = true;
	
	if(0 == vm->framerate){
		vm->framerate = 25;
		ZH_DEBUG(1,"Load Default Rate!\n");
		return ;
	}
	
	if(vm->prevrate != vm->framerate){
		vm->prevrate = vm->framerate;
		vm->jbefore = jiffies;
		vm->jdelay = HZ/vm->framerate;
		return ;
	}
	
	jnext = vm->jbefore + vm->jdelay;
	if(time_is_after_jiffies(jnext)) {
		v4l2_sleep(vm,jnext - jiffies);
		vm->jbefore = jnext;
		return ;
	}		
}

static void framebuf_to_baddr(struct videobuf_buffer *vb, unsigned char  *data, int size)
{
	int i;
	struct page *pg;
	unsigned char * vaddr;	

	for(i = 0; i < vb->dma.nr_pages; i++) {
		pg = pfn_to_page(vb->dma.sglist[i].dma_address >> PAGE_SHIFT);
		vaddr = kmap_atomic(pg,KM_BOUNCE_READ) + vb->dma.sglist[i].offset;
		memcpy(vaddr, data, vb->dma.sglist[i].length);
		kunmap_atomic(vaddr, KM_BOUNCE_READ);
		size -= vb->dma.sglist[i].length;
		data += vb->dma.sglist[i].length;
		if(size <= 0)
			return ;
	}
}

static void V4l2_ProcessOneFrame(Vmanufactory *vm)
{
	unsigned long size;
	unsigned long flags = 0;
	struct videobuf_buffer *vb;	

	spin_lock_irqsave(&vm->listlock, flags);
	if (list_empty(&vm->actives)) {	
		ZH_DEBUG(1,"actives list is empty ! \n");
		goto out;
	}

	vb = list_entry(vm->actives.next, struct videobuf_buffer, queue);
	if (!waitqueue_active(&vb->done)){	
		ZH_DEBUG(1,"No process wait ! \n");
		goto out;		
	}

	list_del(&vb->queue);	
	spin_unlock_irqrestore(&vm->listlock, flags);

	vm->show.ripebuf = vm->frame_buf;//videobuf_to_vmalloc(vb);
	vm->show.stride  = vb->width  * vm->show.depth >> 3;
	vm->show.lines   = vb->height;

	if(vm->show.ripebuf == NULL)
		printk(KERN_INFO "vm->show.ripebuf is NULL !! \n");

	size = vm->show.stride * vm->show.lines;
	if ((size == vm->frame_size) && vm->frame_buf){
		XI102XE_ProcessOneFrame(vm->dev,vm->show.ripebuf, vm->channal);
		framebuf_to_baddr(vb, vm->show.ripebuf, size);
	}
	
	vb->field_count++;
	vb->state = STATE_DONE;
	wake_up(&vb->done);

	return;
out:
	spin_unlock_irqrestore(&vm->listlock, flags);
	return;
}
/******************************************************************************
							extern ops
******************************************************************************/

void Vmanufactory_Init(Vmanufactory *vm, int channal, void *dev)
{
	vm->channal = channal;
	vm->dev = dev;
	vm->framerate = 0;
	vm->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	vm->health = true;
	vm->run = false;

	vm->pthread = NULL;
	
	spin_lock_init(&vm->listlock);
	INIT_LIST_HEAD(&vm->actives);	
	init_waitqueue_head(&vm->wq);
}

int Vmanufactory_Start(Vmanufactory *vm,int(*threadfn)(void *data))
{	
	ZH_DEBUG(1,"vm starting ... \n");
	XI102XE_StartVideoCapture(vm->dev, vm->channal);
	
	vm->pthread = kthread_run(threadfn, vm, "XI102XE_V4L2_THREAD");
	if (IS_ERR(vm->pthread)) {
		return PTR_ERR(vm->pthread);
	}
	
	wake_up_interruptible(&vm->wq);
	
	ZH_DEBUG(1,"vm started\n");

	return 0;	
}

void Vmanufactory_Stop(Vmanufactory *vm)
{
	ZH_DEBUG(1,"vm stopping ... \n");
	if (vm->pthread)
		kthread_stop(vm->pthread);

	vm->pthread = NULL;
	
	XI102XE_StopVideoCapture(vm->dev, vm->channal);
	ZH_DEBUG(1,"vm stop\n");
}

int v4l2_kthread_machine(void *data)
{
	Vmanufactory *vm = data;

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
	set_freezable();
#endif	
	
	while(0 == vm->show.width) {
		v4l2_sleep(vm, HZ/5);
		if (kthread_should_stop())
			return 0;
		ZH_DEBUG(1,"width is %d \n", vm->show.width);
	}
	
	v4l2_framerate_adjust(vm,false);
	
	while(true) {
		if (kthread_should_stop())
			break;
				
		V4l2_ProcessOneFrame(vm);		
		v4l2_framerate_adjust(vm,true);
	}

	return 0;
}

/******************************************************************************
							buffer ops
******************************************************************************/
static void v4l2_buffer_release(struct videobuf_queue *vq, struct videobuf_buffer *vb);
static int v4l2_buffer_setup(struct videobuf_queue *vq, unsigned int *count, unsigned int *size)
{
	Vmanufactory *vm = vq->priv_data;
	*size = vm->show.width * \
			vm->show.height * \
			vm->show.depth >> 3;	

	if (0 == *count)
		*count = 32;

	while (*size * *count > VIDEO_BUF_MAX_M * 1024 * 1024)
		(*count)--;
		
	ZH_DEBUG(3,"Buffer count: %d \n", *count);
	ZH_DEBUG(3,"Buffer info:\n");
	ZH_DEBUG(3, "	Width: %d\n", vm->show.width);
	ZH_DEBUG(3, "	Height: %d\n", vm->show.height);
	ZH_DEBUG(3, "	Depth: %d\n", vm->show.depth);
	
	return 0;
}

static int v4l2_buffer_prepare(	struct videobuf_queue *vq, struct videobuf_buffer *vb,
								enum v4l2_field field)
{
	int rc;
	Vmanufactory *vm = vq->priv_data;

	if (vm->show.width  < 48 || vm->show.width  > 1920 ||
	    vm->show.height < 32 || vm->show.height > 1080)
		return -EINVAL;

	vb->size =  vm->show.width * \
				vm->show.height * \
				vm->show.depth >> 3;
	
	//if(0 != vb->baddr)
		//printk(KERN_INFO "Err : vb->baddr \n");
			
	if (0 != vb->baddr && vb->bsize < vb->size)
		return -EINVAL;
		
	vb->width  = vm->show.width;
	vb->height = vm->show.height;
	vb->bytesperline = vm->show.width * vm->show.depth >> 3;
	vb->field  = field;
	
	if (STATE_NEEDS_INIT == vb->state) {
		rc = videobuf_iolock(vq, vb, NULL);
		if (rc < 0)
			goto fail;
	}

	vb->state = STATE_PREPARED;
	return 0;

fail:
	v4l2_buffer_release(vq, vb);
	return rc;
}

static void v4l2_buffer_queue(struct videobuf_queue *vq, struct videobuf_buffer *vb)
{
	Vmanufactory *vm = vq->priv_data;
	
	vb->state = STATE_QUEUED;
	list_add_tail(&vb->queue, &vm->actives);
}

static void v4l2_buffer_release(struct videobuf_queue *vq, struct videobuf_buffer *vb)
{
	if (in_interrupt())
		BUG();

	videobuf_waiton(vb,0,0);

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
	videobuf_vmalloc_free(vb);
#else
	videobuf_dma_unmap(vq, &vb->dma);
	videobuf_dma_free(&vb->dma);
#endif 

	vb->state = STATE_NEEDS_INIT;
}

/******************************************************************************
							ioctl ops
******************************************************************************/
static int vidioc_querycap(struct file *file, void  *priv,
					struct v4l2_capability *cap)
{
	strcpy(cap->driver, "XI102XE");
	strcpy(cap->card, "XI102XE");
	cap->capabilities = V4L2_CAP_VIDEO_CAPTURE|
						V4L2_CAP_STREAMING    |
						V4L2_CAP_READWRITE;

	return 0;
}

static int vidioc_enum_fmt_vid_cap(struct file *file, void  *priv,
					struct v4l2_fmtdesc *f)
{
	const VFmt *fmt;
	
	if (f->index >= ARRAY_SIZE(formats))
		return -EINVAL;

	fmt = &formats[f->index];

	strlcpy(f->description, fmt->name, sizeof(f->description));
	f->pixelformat = fmt->fourcc;

	return 0;
}

static int vidioc_g_fmt_vid_cap(struct file *file, void *priv,
					struct v4l2_format *f)
{
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	f->fmt.pix.width        = vm->show.width;
	f->fmt.pix.height       = vm->show.height;;
	f->fmt.pix.field        = vm->vb_vidq.field;
	f->fmt.pix.pixelformat  = vm->show.pixelformat;
	f->fmt.pix.bytesperline = (vm->show.width * vm->show.depth) >> 3;
	f->fmt.pix.sizeimage 	= (vm->show.width * vm->show.height * vm->show.depth) >> 3;
	return (0);
}

static int vidioc_try_fmt_vid_cap(struct file *file, void *priv,
			struct v4l2_format *f)
{
	const VFmt 	*fmt;
	enum v4l2_field field;
	
	unsigned int maxw = 1920;
	unsigned int maxh = 1080;

	fmt = get_format(f);
	if (!fmt) {
		printk(KERN_WARNING "Format Mismatch!\n");
		return -EINVAL;
	}

	field = f->fmt.pix.field;
	if (field == V4L2_FIELD_ANY) {
		field = V4L2_FIELD_NONE;
	} else if (V4L2_FIELD_NONE != field) {
		return -EINVAL;
	}

	f->fmt.pix.field = field;
	if (f->fmt.pix.height < 32)
		f->fmt.pix.height = 32;
	if (f->fmt.pix.height > maxh)
		f->fmt.pix.height = maxh;
	if (f->fmt.pix.width < 48)
		f->fmt.pix.width = 48;
	if (f->fmt.pix.width > maxw)
		f->fmt.pix.width = maxw;
	f->fmt.pix.width &= ~0x03;
	f->fmt.pix.bytesperline =
		(f->fmt.pix.width * fmt->depth) >> 3;
	f->fmt.pix.sizeimage =
		f->fmt.pix.height * f->fmt.pix.bytesperline;

	return 0;
}

static COLOR_SPACE V4L2FMT2CSOUT(uint32_t	pixelformat)
{
	switch(pixelformat) {
	case V4L2_PIX_FMT_GREY:
		return CS_GRAY8;
	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB555X:
		return CS_RGB15;
	case V4L2_PIX_FMT_RGB565:
	case V4L2_PIX_FMT_RGB565X:
		return CS_RGB16;
	case V4L2_PIX_FMT_BGR24:
	case V4L2_PIX_FMT_RGB24:
		return CS_RGB24;
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB32:
		return CS_RGB32;
	case V4L2_PIX_FMT_YUYV:
		return CS_YUYV;
	case V4L2_PIX_FMT_YVYU:
		return CS_YVYU;
	case V4L2_PIX_FMT_UYVY:
		return CS_UYVY;
	case V4L2_PIX_FMT_VYUY:
		return CS_VYUY;		
		
	default:
		return CS_UNKNOWN;
	}
}

static int vidioc_s_fmt_vid_cap(struct file *file, void *priv,
					struct v4l2_format *f)
{
	unsigned int size;
	const VFmt 	*fmt;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;
	struct videobuf_queue *q = &vm->vb_vidq;

	int ret = vidioc_try_fmt_vid_cap(file, fh, f);
	if (ret < 0) {
		printk(KERN_WARNING "Can't set fmt!\n");
		return (ret);
	}	
	//mutex_lock(&q->vb_lock);

	if (videobuf_queue_is_busy(q)) {
		ret = -EBUSY;
		goto out;
	}

	fmt = get_format(f);
	vm->show.pixelformat = fmt->fourcc;
	vm->show.depth 		 = fmt->depth;
	vm->show.width 		 = f->fmt.pix.width;
	vm->show.height		 = f->fmt.pix.height;
	vm->vb_vidq.field 	 = f->fmt.pix.field;
	vm->type 			 = f->type;
	
	ZH_DEBUG(3,"Show video with:\n");
	ZH_DEBUG(3,"	width: %d\n", vm->show.width);
	ZH_DEBUG(3,"	height:%d\n",vm->show.height);
	ZH_DEBUG(3,"	format:%d\n",fmt->fourcc);
	ZH_DEBUG(3,"	depth: %d\n",vm->show.depth);
	
	size = vm->show.width * vm->show.height * vm->show.depth >> 3;

	if(NULL == vm->frame_buf) {
		vm->frame_size = size;
		vm->frame_buf = vmalloc_32(size);
		if (vm->frame_buf == NULL) {
			ret = -ENOMEM;
			goto out;
		}
	}

	if(size != vm->frame_size) {
		vfree(vm->frame_buf);
		vm->frame_size = size;
		vm->frame_buf = vmalloc_32(size);
		if (vm->frame_buf == NULL) {
			ret = -ENOMEM;
			goto out;
		}
	}	
	
	XI102XE_SetOutputFormat(
					vm->dev,
					vm->channal,
					true, 
					V4L2FMT2CSOUT(vm->show.pixelformat),
					vm->show.width,
					vm->show.height
					);

	ret = 0;
out:
	//mutex_unlock(&q->vb_lock);
	
	return ret;
}

static int vidioc_reqbufs(struct file *file, void *priv,
			  struct v4l2_requestbuffers *p)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	ret = (videobuf_reqbufs(&vm->vb_vidq, p));

	return ret;
}

static int vidioc_querybuf(struct file *file, void *priv, struct v4l2_buffer *p)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	ret = (videobuf_querybuf(&vm->vb_vidq, p));

	return ret;
}

static int vidioc_qbuf(struct file *file, void *priv, struct v4l2_buffer *p)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	ret = (videobuf_qbuf(&vm->vb_vidq, p));

	return ret;
}

static int vidioc_dqbuf(struct file *file, void *priv, struct v4l2_buffer *p)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;
		
	ret = (videobuf_dqbuf(&vm->vb_vidq, p,
					file->f_flags & (O_NONBLOCK)));

	return ret;
}

static int vidioc_enum_input(struct file *file, void *priv,
				struct v4l2_input *inp)
{
	//XIVdrv *vdrv = video_drvdata(file);	
	//XIVfh *fh 	= priv;
	//XIVdrv *vdrv    = fh->vdrv;

	if (inp->index != 0)
		return -EINVAL;

	inp->type = V4L2_INPUT_TYPE_CAMERA;
	strcpy(inp->name, "XI102XE");
	inp->std = V4L2_STD_NTSC_M;

	return 0;
}

static int vidioc_g_input(struct file *file, void *priv, unsigned int *i)
{
	*i = 0;
	return 0;
}

static int vidioc_s_input(struct file *file, void *priv, unsigned int i)
{
	if (i > 0)
		return -EINVAL;	

	return (0);
}

static int vidioc_queryctrl(struct file *file, void *priv,
			    struct v4l2_queryctrl *qc)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(xi_qctrl); i++)
		if (qc->id && qc->id == xi_qctrl[i].id) {
			memcpy(qc, &(xi_qctrl[i]),sizeof(*qc));
			return (0);
		}

	return -EINVAL;
}

static int vidioc_g_ctrl(struct file *file, void *priv,
			 struct v4l2_control *ctrl)
{
	int i;
	//XIVdrv *vdrv = video_drvdata(file);	
	XIVfh *fh 	= priv;
	XIVdrv *vdrv    = fh->vdrv;

	for (i = 0; i < ARRAY_SIZE(xi_qctrl); i++)
		if (ctrl->id == xi_qctrl[i].id) {
			ctrl->value = vdrv->configure[i];
			return (0);
		}

	return -EINVAL;
}

/*
static int xi_setcfg(XIVdrv *drv, XI_KEY key, int value)
{
	XI_INFO info;
	
	if(generate_xi_info(&info, drv->romid, drv->rlg, true, drv->channal)) {
		Tree *t = get_tree_by_info(&info);
		
		if(t) {
			bool ret;
			ZH_DEBUG(0, "Get the tree by info \n");
			ret = set_s32_by_key(t, key, value);
			
			if(ret)
				ZH_DEBUG(0, "set value by key\n");
		}
	}
	return 0;	
}
*/
#define xi_setcfg(args...) do {} while(0)

static int vidioc_s_ctrl(struct file *file, void *priv,
				struct v4l2_control *ctrl)
{
	/*
	int i, channal;
	//XIVdrv *drv = video_drvdata(file);
	XIVfh *fh 	= priv;
	XIVdrv *drv    = fh->vdrv;
	
	long val = ctrl->value;
	
	for (i = 0; i < ARRAY_SIZE(xi_qctrl); i++)
		if (ctrl->id == xi_qctrl[i].id) {
			
			if (ctrl->value < xi_qctrl[i].minimum
			    || ctrl->value > xi_qctrl[i].maximum) {
					return (-ERANGE);
				}				
			
			drv->configure[i] = ctrl->value;
			channal = drv->channal;
			
			switch (ctrl->id) {
				case V4L2_CID_BRIGHTNESS:
					XI102XE_SetBrightness(drv->priv, &val, drv->channal);
					xi_setcfg(drv, XI_BRIGHTNESS, val);
					break;

				case V4L2_CID_CONTRAST:
					XI102XE_SetContrast(drv->priv, &val, drv->channal);
					xi_setcfg(drv, XI_CONTRAST, val);
					break;

				case V4L2_CID_SATURATION:
					XI102XE_SetSaturation(drv->priv, &val, drv->channal);
					xi_setcfg(drv, XI_SATURATION, val);
					break;

				case V4L2_CID_HUE:
					XI102XE_SetHue(drv->priv, &val, drv->channal);
					xi_setcfg(drv, XI_HUE, val);
					break;
		
				case XI_CID_CUSTOM_VFLIP:
					XI102XE_SetVerticalFlip(drv->priv, val, drv->channal);
					xi_setcfg(drv, XI_VFLIP, val);
					break;
		
				case XI_CID_CUSTOM_HFLIP:
					XI102XE_SetHorizontalFlip(drv->priv, val, drv->channal);
					xi_setcfg(drv, XI_HFLIP, val);
					break;
		
				case XI_CID_CUSTOM_AUTO_TRIGGER:
					XI102XE_SetHDCaptureTriggerAutoPosition(drv->priv, val, drv->channal);
					xi_setcfg(drv, XI_AUTO_TRIGGER, val);
					break;

				case XI_CID_CUSTOM_DEINTERLACE:
					XI102XE_SetDeinterlaceMethod(drv->priv, val, drv->channal);
					xi_setcfg(drv, XI_DEINTERLACE, val);
					break;
		
				case XI_CID_CUSTOM_SCALE:
					XI102XE_SetScaleMode(drv->priv, val, drv->channal);
					xi_setcfg(drv, XI_SCALE, val);
					break;
		
				case XI_CID_CUSTOM_PHASE:
					if(val < -31)
						val = -31;
					if(val > 31)
						val = 31;
					XI102XE_SetCapturePhase(drv->priv, &val, drv->channal);
					xi_setcfg(drv, XI_PHASE, val);
					break;

				default:
					return -EINVAL;
			}
			return (0);
		}
	
	return -EINVAL;
	 */
	return 0;
}

static int vidioc_streamon(struct file *file, void *priv, enum v4l2_buf_type i)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	if (vm->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;
	if (i != vm->type)
		return -EINVAL;

	ret = videobuf_streamon(&vm->vb_vidq);
	return ret;
}

static int vidioc_streamoff(struct file *file, void *priv, enum v4l2_buf_type i)
{
	int ret;
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	if (vm->type != V4L2_BUF_TYPE_VIDEO_CAPTURE) {
		ZH_DEBUG(0,"Stream off Error\n");
		return -EINVAL;
	}
	if (i != vm->type) {
		ZH_DEBUG(0,"Stream off type Error\n");
		return -EINVAL;
	}	

	ret = videobuf_streamoff(&vm->vb_vidq);

	return ret;
}
/*
static int vidioc_g_parm(struct file *file, void *priv,
			 struct v4l2_streamparm *sp)
{
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;
		
	if (sp->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;

	sp->parm.capture.timeperframe.denominator = 
								vm->framerate;	
	sp->parm.capture.timeperframe.numerator = 1;
	
	ZH_DEBUG(1,"Current rate is :%d \n",vm->framerate);
	
	return 0;
}

static int vidioc_s_parm(struct file *file, void *priv,
			 struct v4l2_streamparm *sp)
{
	XIVfh *fh = priv;
	Vmanufactory *vm = &fh->vm;

	if (sp->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;

	vm->framerate = sp->parm.capture.timeperframe.denominator/
					sp->parm.capture.timeperframe.numerator;
					
	ZH_DEBUG(1,"Set rate with :%d \n",vm->framerate);
	
	return 0;
}
*/


static int xi_map_sg(void *dev, struct scatterlist *sg, int nents,
		       int direction)
{
	int i;

	BUG_ON(direction == DMA_NONE);

	for (i = 0; i < nents; i++ ) {
		BUG_ON(!sg[i].page);

		sg[i].dma_address = page_to_phys(sg[i].page) + sg[i].offset;
	}

	return nents;
}

static int xi_unmap_sg(void *dev,struct scatterlist *sglist,int nr_pages,
			 int direction)
{
	return 0;
}

static int xi_dma_sync_sg(void *dev,struct scatterlist *sglist, int nr_pages,
			    int direction)
{
	return 0;
}

/******************************************************************************
							fops ops
******************************************************************************/
static struct videobuf_queue_ops xi_video_qops = {
	.buf_setup      = v4l2_buffer_setup,
	.buf_prepare    = v4l2_buffer_prepare,
	.buf_queue      = v4l2_buffer_queue,
	.buf_release    = v4l2_buffer_release,
	.vb_map_sg      = xi_map_sg,
	.vb_dma_sync_sg = xi_dma_sync_sg,
	.vb_unmap_sg    = xi_unmap_sg,
};

static int xi_open(struct inode *inode, struct file *file)
{
	struct video_device *dev = video_devdata(file);

	//XIVdrv	*drv = video_drvdata(file);
	XIVfh 	*fh 	= NULL;
	XIVdrv	*drv 	= dev->priv;
	
	ZH_DEBUG(1,"Open ... \n");
	mutex_lock(&drv->mutex);
	if(drv->used) {
		mutex_unlock(&drv->mutex);
		return -EBUSY;
	} else {
		drv->used++;
		mutex_unlock(&drv->mutex);
	}
	
	fh = kzalloc(sizeof(*fh), GFP_KERNEL);
	if(NULL == fh){
		mutex_lock(&drv->mutex);
		drv->used--;
		mutex_unlock(&drv->mutex);	
		return -ENOMEM;
	}
	
	Vmanufactory_Init(&fh->vm,drv->channal,drv->priv);

	fh->vm.framerate = 25;
	fh->vm.show.width = 1080;
	fh->vm.show.height = 720;
	fh->vm.show.pixelformat = V4L2_PIX_FMT_YUYV;
	
	fh->vdrv = drv;
	file->private_data = fh;	
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
	videobuf_queue_vmalloc_init(&fh->vm.vb_vidq, 
				&xi_video_qops,
				NULL, 
				&fh->vm.listlock,
				fh->vm.type, 
				V4L2_FIELD_NONE,
				sizeof(struct videobuf_buffer), 
				&fh->vm/*,
				NULL*/);
#else
	videobuf_queue_init(&fh->vm.vb_vidq, 
				&xi_video_qops,
				NULL, 
				&fh->vm.listlock,
				fh->vm.type, 
				V4L2_FIELD_NONE,
				sizeof(struct videobuf_buffer), 
				&fh->vm);
#endif 
	
	ZH_DEBUG(7,"Open info:\n");
	ZH_DEBUG(7,"	signal width: %d \n",fh->vm.show.width);
	ZH_DEBUG(7,"	signal height: %d \n", fh->vm.show.height);
	ZH_DEBUG(7,"	signal rate: %d \n", fh->vm.framerate);

	return Vmanufactory_Start(&fh->vm,v4l2_kthread_machine);
}

static int xi_close(struct inode *inode, struct file *file)
{
	XIVfh *fh = file->private_data;
	XIVdrv *v4l2 = fh->vdrv;
	Vmanufactory *vm = &fh->vm;	

	ZH_DEBUG(1,"xi Close\n");
	Vmanufactory_Stop(vm);

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
	videobuf_stop(&fh->vm.vb_vidq);
#endif 

	videobuf_mmap_free(&fh->vm.vb_vidq);
	
	kfree(fh);

	mutex_lock(&v4l2->mutex);
	v4l2->used = 0;
	mutex_unlock(&v4l2->mutex);

	if(vm->frame_buf) {
		vfree(vm->frame_buf);
		vm->frame_size = 0;
		vm->frame_buf = NULL;
	}	
	
	return 0;
}

static ssize_t xi_read(struct file *file, char __user *data, size_t count, loff_t *ppos)
{
	XIVfh *fh = file->private_data;

	if (fh->vm.type == V4L2_BUF_TYPE_VIDEO_CAPTURE) {
		//return videobuf_read_stream(&fh->vm.vb_vidq, 
		//			data, count, ppos, 0,
		//			file->f_flags & O_NONBLOCK);
		return videobuf_read_one(&fh->vm.vb_vidq, data, count, ppos,
					file->f_flags & O_NONBLOCK);
	}
	return 0;
}

static unsigned int xi_poll(struct file *file, struct poll_table_struct *wait)
{
	XIVfh *fh = file->private_data;
	struct videobuf_queue *q = &fh->vm.vb_vidq;
	struct videobuf_buffer *vb;

	if (V4L2_BUF_TYPE_VIDEO_CAPTURE != fh->vm.type)
		return POLLERR;

	if (list_empty(&q->stream))
		return POLLERR;

	vb = list_entry(q->stream.next, struct videobuf_buffer, stream);

	poll_wait(file, &vb->done, wait);
	if (vb->state == STATE_DONE ||
	    vb->state == STATE_ERROR)
		return POLLIN|POLLRDNORM;
	return 0;



/*
	if (V4L2_BUF_TYPE_VIDEO_CAPTURE != fh->vm.type)
		return POLLERR;

	ret =  videobuf_poll_stream(file, q, wait);
	return ret;
*/
}

static int xi_mmap(struct file *file, struct vm_area_struct *vma)
{
	int ret;
	XIVfh  *fh = file->private_data;

	ret = videobuf_mmap_mapper(&fh->vm.vb_vidq, vma);
	return ret;
}

/**************************************************************
 * 					common struct
 **************************************************************/ 
static struct file_operations xi_fops = {
	.owner		= THIS_MODULE,
	.open           = xi_open,
	.release        = xi_close,
	.read           = xi_read,
	.poll		= xi_poll,
	.ioctl          = video_ioctl2,
	.mmap           = xi_mmap,
	.llseek         = no_llseek,
};

#define V4L2_STD_PRIVATE_MASK (V4L2_STD_NTSC_M | \
						 V4L2_STD_NTSC_443 | \
						 V4L2_STD_PAL_D | \
						 V4L2_STD_PAL_M | \
						 V4L2_STD_PAL_Nc | \
						 V4L2_STD_PAL_60 | \
						 V4L2_STD_SECAM_B | \
						 V4L2_STD_SECAM_D)

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,22)
static struct v4l2_tvnorm tvnorms[] = {
	{
		.name      = "NTSC-M",
		.id        = V4L2_STD_NTSC_M,
	}
};
#endif 

static struct video_device xi_template = {
	.name				= "XI102XE",
	.type				= VID_TYPE_CAPTURE,
//	.hardware			= 0,
	.fops           	= &xi_fops,
	.minor				= -1,
	.vidioc_querycap      = vidioc_querycap,
	.vidioc_enum_fmt_cap  = vidioc_enum_fmt_vid_cap,
	.vidioc_g_fmt_cap     = vidioc_g_fmt_vid_cap,
	.vidioc_try_fmt_cap   = vidioc_try_fmt_vid_cap,
	.vidioc_s_fmt_cap     = vidioc_s_fmt_vid_cap,
	.vidioc_reqbufs       = vidioc_reqbufs,
	.vidioc_querybuf      = vidioc_querybuf,
	.vidioc_qbuf          = vidioc_qbuf,
	.vidioc_dqbuf         = vidioc_dqbuf,
	.vidioc_enum_input    = vidioc_enum_input,
	.vidioc_g_input       = vidioc_g_input,
	.vidioc_s_input       = vidioc_s_input,
	.vidioc_queryctrl     = vidioc_queryctrl,
	.vidioc_g_ctrl        = vidioc_g_ctrl,
	.vidioc_s_ctrl        = vidioc_s_ctrl,
	.vidioc_streamon      = vidioc_streamon,
	.vidioc_streamoff     = vidioc_streamoff,
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,22)
	.tvnorms              = V4L2_STD_PRIVATE_MASK,
	.current_norm         = V4L2_STD_PAL_M,
#else
	.tvnorms              = tvnorms,
	.tvnormsize           = ARRAY_SIZE(tvnorms),
#endif

};
/******************************************************************************
							module ops
******************************************************************************/
static int video_nr 	= -1;	
module_param(video_nr, uint, 0444);
MODULE_PARM_DESC(video_nr, "video iminor start number");

/*
static void xi_config_load(XIVdrv *drv)
{
	XI_INFO info;

	if(generate_xi_info(&info, drv->romid, drv->rlg, false, drv->channal)) {
		Tree * t = get_tree_by_info(&info);
		if(t) {
			long value, i = 0;;
			
			if(get_s32_by_key(t, XI_BRIGHTNESS, &value)) {	
				XI102XE_SetBrightness(drv->priv, &value);				
				drv->configure[i] = value;			
			}			
			i++;
			
			if(get_s32_by_key(t, XI_CONTRAST, &value)) {
				XI102XE_SetContrast(drv->priv, &value);
				drv->configure[i] = value;	
			}
			i++;
			
			if(get_s32_by_key(t, XI_SATURATION, &value)) {
				XI102XE_SetSaturation(drv->priv, &value);
				drv->configure[i] = value;	
			}
			i++;
			
			if(get_s32_by_key(t, XI_HUE, &value)) {			
				XI102XE_SetHue(drv->priv, &value);
				drv->configure[i] = value;				
			}
			i++;
			
			if(get_s32_by_key(t, XI_VFLIP, &value)) {			
				XI102XE_SetVerticalFlip(drv->priv, value);
				drv->configure[i] = value;		
			}
			i++;
			
			if(get_s32_by_key(t, XI_HFLIP, &value)) {			
				XI102XE_SetHorizontalFlip(drv->priv, value);
				drv->configure[i] = value;		
			}
			i++;
			
			if(get_s32_by_key(t, XI_AUTO_TRIGGER, &value)) {			
				XI102XE_SetHDCaptureTriggerAutoPosition(drv->priv, value);
				drv->configure[i] = value;				
			}
			i++;

			if(get_s32_by_key(t, XI_DEINTERLACE, &value)) {			
				XI102XE_SetDeinterlaceMethod(drv->priv, value);
				drv->configure[i] = value;			
			}
			i++;
			
			if(get_s32_by_key(t, XI_SCALE, &value)) {			
				XI102XE_SetScaleMode(drv->priv, value);
				drv->configure[i] = value;			
			}
			i++;
			
			if(get_s32_by_key(t, XI_PHASE, &value)) {			
				XI102XE_SetCapturePhase(drv->priv, &value);
				drv->configure[i] = value;				
			}
			i++;
		}
	}
}
*/
#define xi_config_load(args...) do {} while(0)

int __init v4l2_instance_create(void *dev,int inst)
{	
	XIVdrv *vdrv;
	struct video_device *vfd;
	int i, ret = -ENOMEM;
	
	vdrv = kzalloc(sizeof(*vdrv), GFP_KERNEL);
	if (!vdrv)
		return -ENOMEM;

	//snprintf(vdrv->v4l2.name, sizeof(vdrv->v4l2.name),
	//		"%s-%03d", "XI102XE", inst);
	//ret = v4l2_device_register(NULL, &vdrv->v4l2);
	//if (ret)
	//	goto free_dev;

	mutex_init(&vdrv->mutex);

	vfd = video_device_alloc();
	if (!vfd)
		goto unreg_dev;

	*vfd = xi_template;

	ret = video_register_device(vfd, VFL_TYPE_GRABBER, video_nr);
	if (ret < 0)
		goto rel_vdev;

	video_set_drvdata(vfd, vdrv);

	vdrv->configure = kzalloc(sizeof(int) * ARRAY_SIZE(xi_qctrl), GFP_KERNEL);
	if(NULL ==  vdrv->configure)
		goto rel_vdev;
	 
	for (i = 0; i < ARRAY_SIZE(xi_qctrl); i++)
		vdrv->configure[i] = xi_qctrl[i].default_value;

	list_add_tail(&vdrv->vlist, XI102XE_GetV4l2ListHead(dev));

	if (video_nr >= 0)
		video_nr++;

	vdrv->used 		= 0;
	vdrv->vfd 		= vfd;
	vdrv->channal 	= inst;
	vdrv->priv		= dev;
	
	//add for ROMID
	vdrv->romid = XI102XE_GetROMID(dev,&vdrv->rlg);
	
	xi_config_load(vdrv);

	return 0;

rel_vdev:
	video_unregister_device(vfd);
	video_device_release(vfd);
unreg_dev:
	//v4l2_device_unregister(&vdrv->v4l2);
//free_dev:
	kfree(vdrv);
	return ret;
}

int xi_v4l2_release(void *dev)
{
	XIVdrv *vdrv;
	struct list_head *list;
	struct list_head *head;
	
	head = XI102XE_GetV4l2ListHead(dev);

	while (!list_empty(head)) {
		list = head->next;
		list_del(list);
		vdrv = list_entry(list, XIVdrv, vlist);
		
		video_unregister_device(vdrv->vfd);
		//v4l2_device_unregister(&vdrv->v4l2);		
		
		kfree(vdrv->configure);
		kfree(vdrv);
	}

	return 0;
}

int xi_v4l2_init(void *dev)
{
	int ret = -ENOMEM, i;

	for (i = 0; i < XI102XE_VIDEO_DEVICE_COUNT; i++) {
		ret = v4l2_instance_create(dev,i);
		if (ret) {			
			break;
		}
	}
	printk(KERN_INFO "n_v4l2s counts = %d\n",i);
	return ret;
}
