#pragma once

#include <linux/version.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/mutex.h>
#include <linux/videodev2.h>
#include <linux/kthread.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/vmalloc.h>

#include <media/video-buf.h>
#include <media/v4l2-dev.h>
#include <media/v4l2-common.h>

#include "OSAbstract/OSAbstract.h"
#include "Drivers/XI102XE_Device.h"

#define VIDEO_BUF_MAX_M	16

typedef struct _ShowInfo {	
	uint32_t 	width;
	uint32_t 	height;
	uint32_t	pixelformat;
	uint32_t	depth;

	uint8_t 	*ripebuf;
	uint32_t 	stride;
	uint32_t  	lines;
}VShowInfo;

typedef struct _VManufactory {
	uint32_t				channal;
	uint32_t				framerate;	
	VShowInfo				show;
	
	spinlock_t				listlock;
	struct videobuf_queue	vb_vidq;	
	enum v4l2_buf_type		type;
	
	struct task_struct 		*pthread;	
	struct list_head 		actives; 
	wait_queue_head_t		wq;
	void *					dev;
	
	unsigned long 			jbefore;
	unsigned long 			jdelay;
	unsigned long 			prevrate;
	bool 					run;
	bool 					health;

	//add for baddr
	unsigned char 	*frame_buf; //bytes
	unsigned int 	frame_size;
}Vmanufactory;

typedef struct _VFmt {
	char 		*name;
	u32			fourcc;
	int 		depth;	
}VFmt;

struct sg_to_addr {
	int 				pos;
	struct scatterlist 	*sg;
};

struct xi_buffer {
	struct videobuf_buffer 	vb;
	VFmt        			*fmt;
	struct sg_to_addr      	*to_addr;
};


typedef struct _XI_VDrv {
	struct list_head	vlist;
	struct video_device	*vfd;

	struct mutex		mutex;
	unsigned int		used;
	signed int			channal;
	signed int			*configure;	
	void 				*priv;
	
	//add for ROMID
	unsigned char 		*romid;
	unsigned int 		rlg;
}XIVdrv;

typedef struct _XI_Vfh {
	Vmanufactory 		vm;	
	XIVdrv 				*vdrv;
}XIVfh;

int xi_v4l2_init(void *dev);
int xi_v4l2_release(void *dev);

static inline void xi_v4l2_suspend(void *dev)
{	
}

static inline void  xi_v4l2_resume(void *dev)
{	
}
