#pragma once

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/font.h>
#include <linux/mutex.h>
#include <linux/videodev2.h>
#include <linux/kthread.h>
#include <linux/freezer.h>

#include <media/v4l2-ctrls.h>
#include <media/v4l2-fh.h>

#include <media/videobuf-vmalloc.h>
#include <media/v4l2-device.h>
#include <media/v4l2-common.h>
#include <media/v4l2-ioctl.h>

#include "OSAbstract/OSAbstract.h"
#include "Drivers/XI102XE_Device.h"

#define VIDEO_BUF_MAX_M		16
#define MAX_WIDTH 			1920
#define MAX_HEIGHT 			1080

typedef struct _VFmt {
	char 	*name;
	u32		fourcc;
	int 	depth;
}VFmt;

typedef enum _FrameSyscBy {
	FRAME_SYSC_BY_RATE,
	FRAME_SYSC_BY_IRQ
}FrameSyscBy;

typedef struct _VManufactory {
	VideoShowInfo			show;
	
	struct task_struct 		*pthread;
	struct list_head 		actives; 
	wait_queue_head_t		wq;
	int 					channal;
	void *					dev;

	unsigned long 			jbefore;
	unsigned long 			jdelay;
	unsigned long 			prevrate;
	uint32_t				framerate;

	spinlock_t				slock;
	enum v4l2_field			field;
	unsigned long			field_count;
	struct videobuf_queue	vb_vidq;
	enum v4l2_buf_type		type;
}Vmanufactory;

typedef struct _XIVdrv {
	struct v4l2_device	v4l2_dev;
	Vmanufactory		vm;
	
	struct mutex		mutex;
	unsigned int 		used;
	signed int			channal;
	void  *				dev;
	
	struct video_device	*vfd;
	struct list_head	vlist;
}XIVdrv;

int xi_v4l2_init(void *dev);
int xi_v4l2_release(void *dev);
static inline void xi_v4l2_suspend(void *dev)
{
}
static inline void  xi_v4l2_resume(void *dev)
{
}
